# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1.settings import (
    BillingProviderListResponse,
    BillingProviderCreateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBillingProviders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Metronome) -> None:
        billing_provider = client.v1.settings.billing_providers.create(
            billing_provider="aws_marketplace",
            configuration={
                "aws_external_id": "bar",
                "aws_iam_role_arn": "bar",
            },
            delivery_method="direct_to_billing_provider",
        )
        assert_matches_type(BillingProviderCreateResponse, billing_provider, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Metronome) -> None:
        response = client.v1.settings.billing_providers.with_raw_response.create(
            billing_provider="aws_marketplace",
            configuration={
                "aws_external_id": "bar",
                "aws_iam_role_arn": "bar",
            },
            delivery_method="direct_to_billing_provider",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_provider = response.parse()
        assert_matches_type(BillingProviderCreateResponse, billing_provider, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Metronome) -> None:
        with client.v1.settings.billing_providers.with_streaming_response.create(
            billing_provider="aws_marketplace",
            configuration={
                "aws_external_id": "bar",
                "aws_iam_role_arn": "bar",
            },
            delivery_method="direct_to_billing_provider",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_provider = response.parse()
            assert_matches_type(BillingProviderCreateResponse, billing_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Metronome) -> None:
        billing_provider = client.v1.settings.billing_providers.list()
        assert_matches_type(BillingProviderListResponse, billing_provider, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Metronome) -> None:
        billing_provider = client.v1.settings.billing_providers.list(
            next_page="af26878a-de62-4a0d-9b77-3936f7c2b6d6",
        )
        assert_matches_type(BillingProviderListResponse, billing_provider, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Metronome) -> None:
        response = client.v1.settings.billing_providers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_provider = response.parse()
        assert_matches_type(BillingProviderListResponse, billing_provider, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Metronome) -> None:
        with client.v1.settings.billing_providers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_provider = response.parse()
            assert_matches_type(BillingProviderListResponse, billing_provider, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncBillingProviders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncMetronome) -> None:
        billing_provider = await async_client.v1.settings.billing_providers.create(
            billing_provider="aws_marketplace",
            configuration={
                "aws_external_id": "bar",
                "aws_iam_role_arn": "bar",
            },
            delivery_method="direct_to_billing_provider",
        )
        assert_matches_type(BillingProviderCreateResponse, billing_provider, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.settings.billing_providers.with_raw_response.create(
            billing_provider="aws_marketplace",
            configuration={
                "aws_external_id": "bar",
                "aws_iam_role_arn": "bar",
            },
            delivery_method="direct_to_billing_provider",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_provider = await response.parse()
        assert_matches_type(BillingProviderCreateResponse, billing_provider, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.settings.billing_providers.with_streaming_response.create(
            billing_provider="aws_marketplace",
            configuration={
                "aws_external_id": "bar",
                "aws_iam_role_arn": "bar",
            },
            delivery_method="direct_to_billing_provider",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_provider = await response.parse()
            assert_matches_type(BillingProviderCreateResponse, billing_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncMetronome) -> None:
        billing_provider = await async_client.v1.settings.billing_providers.list()
        assert_matches_type(BillingProviderListResponse, billing_provider, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMetronome) -> None:
        billing_provider = await async_client.v1.settings.billing_providers.list(
            next_page="af26878a-de62-4a0d-9b77-3936f7c2b6d6",
        )
        assert_matches_type(BillingProviderListResponse, billing_provider, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.settings.billing_providers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_provider = await response.parse()
        assert_matches_type(BillingProviderListResponse, billing_provider, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.settings.billing_providers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_provider = await response.parse()
            assert_matches_type(BillingProviderListResponse, billing_provider, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v2 import (
    contract_edit_params,
    contract_list_params,
    contract_retrieve_params,
    contract_edit_commit_params,
    contract_edit_credit_params,
    contract_get_edit_history_params,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v2.contract_edit_response import ContractEditResponse
from ...types.v2.contract_list_response import ContractListResponse
from ...types.v2.contract_retrieve_response import ContractRetrieveResponse
from ...types.v2.contract_edit_commit_response import ContractEditCommitResponse
from ...types.v2.contract_edit_credit_response import ContractEditCreditResponse
from ...types.shared_params.commit_specifier_input import CommitSpecifierInput
from ...types.v2.contract_get_edit_history_response import ContractGetEditHistoryResponse
from ...types.shared_params.commit_hierarchy_configuration import CommitHierarchyConfiguration
from ...types.shared_params.spend_threshold_configuration_v2 import SpendThresholdConfigurationV2
from ...types.shared_params.prepaid_balance_threshold_configuration_v2 import PrepaidBalanceThresholdConfigurationV2

__all__ = ["ContractsResource", "AsyncContractsResource"]


class ContractsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContractsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return ContractsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContractsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return ContractsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        contract_id: str,
        customer_id: str,
        as_of_date: Union[str, datetime] | Omit = omit,
        include_balance: bool | Omit = omit,
        include_ledgers: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractRetrieveResponse:
        """
        Gets the details for a specific contract, including contract term, rate card
        information, credits and commits, and more.

        ### Use this endpoint to:

        - Check the duration of a customer's current contract
        - Get details on contract terms, including access schedule amounts for
          commitments and credits
        - Understand the state of a contract at a past time. As you can evolve the terms
          of a contract over time through editing, use the `as_of_date` parameter to
          view the full contract configuration as of that point in time.

        ### Usage guidelines:

        - Optionally, use the `include_balance` and `include_ledger` fields to include
          balances and ledgers in the credit and commit responses. Using these fields
          will cause the query to be slower.

        Args:
          as_of_date: Optional RFC 3339 timestamp. Return the contract as of this date. Cannot be used
              with include_ledgers parameter.

          include_balance: Include the balance of credits and commits in the response. Setting this flag
              may cause the query to be slower.

          include_ledgers: Include commit/credit ledgers in the response. Setting this flag may cause the
              query to be slower. Cannot be used with as_of_date parameter.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/contracts/get",
            body=maybe_transform(
                {
                    "contract_id": contract_id,
                    "customer_id": customer_id,
                    "as_of_date": as_of_date,
                    "include_balance": include_balance,
                    "include_ledgers": include_ledgers,
                },
                contract_retrieve_params.ContractRetrieveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractRetrieveResponse,
        )

    def list(
        self,
        *,
        customer_id: str,
        covering_date: Union[str, datetime] | Omit = omit,
        include_archived: bool | Omit = omit,
        include_balance: bool | Omit = omit,
        include_ledgers: bool | Omit = omit,
        starting_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractListResponse:
        """
        For a given customer, lists all of their contracts in chronological order.

        ### Use this endpoint to:

        - Check if a customer is provisioned with any contract, and at which tier
        - Check the duration and terms of a customer's current contract
        - Power a page in your end customer experience that shows the customer's history
          of tiers (e.g. this customer started out on the Pro Plan, then downgraded to
          the Starter plan).

        ### Usage guidelines:

        Use the `starting_at`, `covering_date`, and `include_archived` parameters to
        filter the list of returned contracts. For example, to list only currently
        active contracts, pass `covering_date` equal to the current time.

        Args:
          covering_date: Optional RFC 3339 timestamp. Only include contracts active on the provided date.
              This cannot be provided if starting_at filter is provided.

          include_archived: Include archived contracts in the response.

          include_balance: Include the balance of credits and commits in the response. Setting this flag
              may cause the response to be slower.

          include_ledgers: Include commit/credit ledgers in the response. Setting this flag may cause the
              response to be slower.

          starting_at: Optional RFC 3339 timestamp. Only include contracts that started on or after
              this date. This cannot be provided if covering_date filter is provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/contracts/list",
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "covering_date": covering_date,
                    "include_archived": include_archived,
                    "include_balance": include_balance,
                    "include_ledgers": include_ledgers,
                    "starting_at": starting_at,
                },
                contract_list_params.ContractListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractListResponse,
        )

    def edit(
        self,
        *,
        contract_id: str,
        customer_id: str,
        add_billing_provider_configuration_update: contract_edit_params.AddBillingProviderConfigurationUpdate
        | Omit = omit,
        add_commits: Iterable[contract_edit_params.AddCommit] | Omit = omit,
        add_credits: Iterable[contract_edit_params.AddCredit] | Omit = omit,
        add_discounts: Iterable[contract_edit_params.AddDiscount] | Omit = omit,
        add_overrides: Iterable[contract_edit_params.AddOverride] | Omit = omit,
        add_prepaid_balance_threshold_configuration: PrepaidBalanceThresholdConfigurationV2 | Omit = omit,
        add_professional_services: Iterable[contract_edit_params.AddProfessionalService] | Omit = omit,
        add_recurring_commits: Iterable[contract_edit_params.AddRecurringCommit] | Omit = omit,
        add_recurring_credits: Iterable[contract_edit_params.AddRecurringCredit] | Omit = omit,
        add_reseller_royalties: Iterable[contract_edit_params.AddResellerRoyalty] | Omit = omit,
        add_scheduled_charges: Iterable[contract_edit_params.AddScheduledCharge] | Omit = omit,
        add_spend_threshold_configuration: SpendThresholdConfigurationV2 | Omit = omit,
        add_subscriptions: Iterable[contract_edit_params.AddSubscription] | Omit = omit,
        allow_contract_ending_before_finalized_invoice: bool | Omit = omit,
        archive_commits: Iterable[contract_edit_params.ArchiveCommit] | Omit = omit,
        archive_credits: Iterable[contract_edit_params.ArchiveCredit] | Omit = omit,
        archive_scheduled_charges: Iterable[contract_edit_params.ArchiveScheduledCharge] | Omit = omit,
        remove_overrides: Iterable[contract_edit_params.RemoveOverride] | Omit = omit,
        uniqueness_key: str | Omit = omit,
        update_commits: Iterable[contract_edit_params.UpdateCommit] | Omit = omit,
        update_contract_end_date: Union[str, datetime, None] | Omit = omit,
        update_contract_name: Optional[str] | Omit = omit,
        update_credits: Iterable[contract_edit_params.UpdateCredit] | Omit = omit,
        update_prepaid_balance_threshold_configuration: contract_edit_params.UpdatePrepaidBalanceThresholdConfiguration
        | Omit = omit,
        update_recurring_commits: Iterable[contract_edit_params.UpdateRecurringCommit] | Omit = omit,
        update_recurring_credits: Iterable[contract_edit_params.UpdateRecurringCredit] | Omit = omit,
        update_scheduled_charges: Iterable[contract_edit_params.UpdateScheduledCharge] | Omit = omit,
        update_spend_threshold_configuration: contract_edit_params.UpdateSpendThresholdConfiguration | Omit = omit,
        update_subscriptions: Iterable[contract_edit_params.UpdateSubscription] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractEditResponse:
        """
        The ability to edit a contract helps you react quickly to the needs of your
        customers and your business.

        ### Use this endpoint to:

        - Encode mid-term commitment and discount changes
        - Fix configuration mistakes and easily roll back packaging changes

        ### Key response fields:

        - The `id` of the edit
        - Complete edit details. For example, if you edited the contract to add new
          overrides and credits, you will receive the IDs of those overrides and credits
          in the response.

        ### Usage guidelines:

        - When you edit a contract, any draft invoices update immediately to reflect
          that edit. Finalized invoices remain unchanged - you must void and regenerate
          them in the UI or API to reflect the edit.
        - Contract editing must be enabled to use this endpoint. Reach out to your
          Metronome representative to learn more.

        Args:
          contract_id: ID of the contract being edited

          customer_id: ID of the customer whose contract is being edited

          add_billing_provider_configuration_update: Update the billing provider configuration on the contract. Currently only
              supports adding a billing provider configuration to a contract that does not
              already have one.

          add_professional_services: This field's availability is dependent on your client's configuration.

          add_subscriptions: Optional list of
              [subscriptions](https://docs.metronome.com/manage-product-access/create-subscription/)
              to add to the contract.

          allow_contract_ending_before_finalized_invoice: If true, allows setting the contract end date earlier than the end_timestamp of
              existing finalized invoices. Finalized invoices will be unchanged; if you want
              to incorporate the new end date, you can void and regenerate finalized usage
              invoices. Defaults to true.

          archive_commits: IDs of commits to archive

          archive_credits: IDs of credits to archive

          archive_scheduled_charges: IDs of scheduled charges to archive

          remove_overrides: IDs of overrides to remove

          uniqueness_key: Optional uniqueness key to prevent duplicate contract edits.

          update_contract_end_date: RFC 3339 timestamp indicating when the contract will end (exclusive).

          update_contract_name: Value to update the contract name to. If not provided, the contract name will
              remain unchanged.

          update_recurring_commits: Edits to these recurring commits will only affect commits whose access schedules
              has not started. Expired commits, and commits with an active access schedule
              will remain unchanged.

          update_recurring_credits: Edits to these recurring credits will only affect credits whose access schedules
              has not started. Expired credits, and credits with an active access schedule
              will remain unchanged.

          update_subscriptions: Optional list of subscriptions to update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/contracts/edit",
            body=maybe_transform(
                {
                    "contract_id": contract_id,
                    "customer_id": customer_id,
                    "add_billing_provider_configuration_update": add_billing_provider_configuration_update,
                    "add_commits": add_commits,
                    "add_credits": add_credits,
                    "add_discounts": add_discounts,
                    "add_overrides": add_overrides,
                    "add_prepaid_balance_threshold_configuration": add_prepaid_balance_threshold_configuration,
                    "add_professional_services": add_professional_services,
                    "add_recurring_commits": add_recurring_commits,
                    "add_recurring_credits": add_recurring_credits,
                    "add_reseller_royalties": add_reseller_royalties,
                    "add_scheduled_charges": add_scheduled_charges,
                    "add_spend_threshold_configuration": add_spend_threshold_configuration,
                    "add_subscriptions": add_subscriptions,
                    "allow_contract_ending_before_finalized_invoice": allow_contract_ending_before_finalized_invoice,
                    "archive_commits": archive_commits,
                    "archive_credits": archive_credits,
                    "archive_scheduled_charges": archive_scheduled_charges,
                    "remove_overrides": remove_overrides,
                    "uniqueness_key": uniqueness_key,
                    "update_commits": update_commits,
                    "update_contract_end_date": update_contract_end_date,
                    "update_contract_name": update_contract_name,
                    "update_credits": update_credits,
                    "update_prepaid_balance_threshold_configuration": update_prepaid_balance_threshold_configuration,
                    "update_recurring_commits": update_recurring_commits,
                    "update_recurring_credits": update_recurring_credits,
                    "update_scheduled_charges": update_scheduled_charges,
                    "update_spend_threshold_configuration": update_spend_threshold_configuration,
                    "update_subscriptions": update_subscriptions,
                },
                contract_edit_params.ContractEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractEditResponse,
        )

    def edit_commit(
        self,
        *,
        commit_id: str,
        customer_id: str,
        access_schedule: contract_edit_commit_params.AccessSchedule | Omit = omit,
        applicable_product_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        applicable_product_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        description: str | Omit = omit,
        hierarchy_configuration: CommitHierarchyConfiguration | Omit = omit,
        invoice_contract_id: str | Omit = omit,
        invoice_schedule: contract_edit_commit_params.InvoiceSchedule | Omit = omit,
        name: str | Omit = omit,
        priority: Optional[float] | Omit = omit,
        product_id: str | Omit = omit,
        rate_type: Literal["LIST_RATE", "COMMIT_RATE"] | Omit = omit,
        specifiers: Optional[Iterable[CommitSpecifierInput]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractEditCommitResponse:
        """Edit specific details for a contract-level or customer-level commit.

        Use this
        endpoint to modify individual commit access schedules, invoice schedules,
        applicable products, invoicing contracts, or other fields.

        ### Usage guidelines:

        - As with all edits in Metronome, draft invoices will reflect the edit
          immediately, while finalized invoices are untouched unless voided and
          regenerated.
        - If a commit's invoice schedule item is associated with a finalized invoice,
          you cannot remove or update the invoice schedule item.
        - If a commit's invoice schedule item is associated with a voided invoice, you
          cannot remove the invoice schedule item.
        - You cannot remove an commit access schedule segment that was applied to a
          finalized invoice. You can void the invoice beforehand and then remove the
          access schedule segment.

        Args:
          commit_id: ID of the commit to edit

          customer_id: ID of the customer whose commit is being edited

          applicable_product_ids: Which products the commit applies to. If applicable_product_ids,
              applicable_product_tags or specifiers are not provided, the commit applies to
              all products.

          applicable_product_tags: Which tags the commit applies to. If applicable_product_ids,
              applicable_product_tags or specifiers are not provided, the commit applies to
              all products.

          description: Updated description for the commit

          hierarchy_configuration: Optional configuration for commit hierarchy access control

          invoice_contract_id: ID of contract to use for invoicing

          name: Updated name for the commit

          priority: If multiple commits are applicable, the one with the lower priority will apply
              first.

          rate_type: If provided, updates the commit to use the specified rate type for current and
              future invoices. Previously finalized invoices will need to be voided and
              regenerated to reflect the rate type change.

          specifiers: List of filters that determine what kind of customer usage draws down a commit
              or credit. A customer's usage needs to meet the condition of at least one of the
              specifiers to contribute to a commit's or credit's drawdown. This field cannot
              be used together with `applicable_product_ids` or `applicable_product_tags`.
              Instead, to target usage by product or product tag, pass those values in the
              body of `specifiers`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/contracts/commits/edit",
            body=maybe_transform(
                {
                    "commit_id": commit_id,
                    "customer_id": customer_id,
                    "access_schedule": access_schedule,
                    "applicable_product_ids": applicable_product_ids,
                    "applicable_product_tags": applicable_product_tags,
                    "description": description,
                    "hierarchy_configuration": hierarchy_configuration,
                    "invoice_contract_id": invoice_contract_id,
                    "invoice_schedule": invoice_schedule,
                    "name": name,
                    "priority": priority,
                    "product_id": product_id,
                    "rate_type": rate_type,
                    "specifiers": specifiers,
                },
                contract_edit_commit_params.ContractEditCommitParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractEditCommitResponse,
        )

    def edit_credit(
        self,
        *,
        credit_id: str,
        customer_id: str,
        access_schedule: contract_edit_credit_params.AccessSchedule | Omit = omit,
        applicable_product_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        applicable_product_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        description: str | Omit = omit,
        hierarchy_configuration: CommitHierarchyConfiguration | Omit = omit,
        name: str | Omit = omit,
        priority: Optional[float] | Omit = omit,
        product_id: str | Omit = omit,
        rate_type: Literal["LIST_RATE", "COMMIT_RATE"] | Omit = omit,
        specifiers: Optional[Iterable[CommitSpecifierInput]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractEditCreditResponse:
        """
        Edit details for a contract-level or customer-level credit.

        ### Use this endpoint to:

        - Extend the duration or the amount of an existing free credit like a trial
        - Modify individual credit access schedules, applicable products, priority, or
          other fields.

        ### Usage guidelines:

        - As with all edits in Metronome, draft invoices will reflect the edit
          immediately, while finalized invoices are untouched unless voided and
          regenerated.
        - You cannot remove an access schedule segment that was applied to a finalized
          invoice. You can void the invoice beforehand and then remove the access
          schedule segment.

        Args:
          credit_id: ID of the credit to edit

          customer_id: ID of the customer whose credit is being edited

          applicable_product_ids: Which products the credit applies to. If both applicable_product_ids and
              applicable_product_tags are not provided, the credit applies to all products.

          applicable_product_tags: Which tags the credit applies to. If both applicable_product_ids and
              applicable_product_tags are not provided, the credit applies to all products.

          description: Updated description for the credit

          hierarchy_configuration: Optional configuration for credit hierarchy access control

          name: Updated name for the credit

          priority: If multiple commits are applicable, the one with the lower priority will apply
              first.

          rate_type: If provided, updates the credit to use the specified rate type for current and
              future invoices. Previously finalized invoices will need to be voided and
              regenerated to reflect the rate type change.

          specifiers: List of filters that determine what kind of customer usage draws down a commit
              or credit. A customer's usage needs to meet the condition of at least one of the
              specifiers to contribute to a commit's or credit's drawdown. This field cannot
              be used together with `applicable_product_ids` or `applicable_product_tags`.
              Instead, to target usage by product or product tag, pass those values in the
              body of `specifiers`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/contracts/credits/edit",
            body=maybe_transform(
                {
                    "credit_id": credit_id,
                    "customer_id": customer_id,
                    "access_schedule": access_schedule,
                    "applicable_product_ids": applicable_product_ids,
                    "applicable_product_tags": applicable_product_tags,
                    "description": description,
                    "hierarchy_configuration": hierarchy_configuration,
                    "name": name,
                    "priority": priority,
                    "product_id": product_id,
                    "rate_type": rate_type,
                    "specifiers": specifiers,
                },
                contract_edit_credit_params.ContractEditCreditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractEditCreditResponse,
        )

    def get_edit_history(
        self,
        *,
        contract_id: str,
        customer_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractGetEditHistoryResponse:
        """List all the edits made to a contract over time.

        In Metronome, you can edit a
        contract at any point after it's created to fix mistakes or reflect changes in
        terms. Metronome stores a full history of all edits that were ever made to a
        contract, whether through the UI, `editContract` endpoint, or other endpoints
        like `updateContractEndDate`.

        ### Use this endpoint to:

        - Understand what changes were made to a contract, when, and by who

        ### Key response fields:

        - An array of every edit ever made to the contract
        - Details on each individual edit - for example showing that in one edit, a user
          added two discounts and incremented a subscription quantity.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/contracts/getEditHistory",
            body=maybe_transform(
                {
                    "contract_id": contract_id,
                    "customer_id": customer_id,
                },
                contract_get_edit_history_params.ContractGetEditHistoryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractGetEditHistoryResponse,
        )


class AsyncContractsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContractsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContractsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContractsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncContractsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        contract_id: str,
        customer_id: str,
        as_of_date: Union[str, datetime] | Omit = omit,
        include_balance: bool | Omit = omit,
        include_ledgers: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractRetrieveResponse:
        """
        Gets the details for a specific contract, including contract term, rate card
        information, credits and commits, and more.

        ### Use this endpoint to:

        - Check the duration of a customer's current contract
        - Get details on contract terms, including access schedule amounts for
          commitments and credits
        - Understand the state of a contract at a past time. As you can evolve the terms
          of a contract over time through editing, use the `as_of_date` parameter to
          view the full contract configuration as of that point in time.

        ### Usage guidelines:

        - Optionally, use the `include_balance` and `include_ledger` fields to include
          balances and ledgers in the credit and commit responses. Using these fields
          will cause the query to be slower.

        Args:
          as_of_date: Optional RFC 3339 timestamp. Return the contract as of this date. Cannot be used
              with include_ledgers parameter.

          include_balance: Include the balance of credits and commits in the response. Setting this flag
              may cause the query to be slower.

          include_ledgers: Include commit/credit ledgers in the response. Setting this flag may cause the
              query to be slower. Cannot be used with as_of_date parameter.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/contracts/get",
            body=await async_maybe_transform(
                {
                    "contract_id": contract_id,
                    "customer_id": customer_id,
                    "as_of_date": as_of_date,
                    "include_balance": include_balance,
                    "include_ledgers": include_ledgers,
                },
                contract_retrieve_params.ContractRetrieveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractRetrieveResponse,
        )

    async def list(
        self,
        *,
        customer_id: str,
        covering_date: Union[str, datetime] | Omit = omit,
        include_archived: bool | Omit = omit,
        include_balance: bool | Omit = omit,
        include_ledgers: bool | Omit = omit,
        starting_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractListResponse:
        """
        For a given customer, lists all of their contracts in chronological order.

        ### Use this endpoint to:

        - Check if a customer is provisioned with any contract, and at which tier
        - Check the duration and terms of a customer's current contract
        - Power a page in your end customer experience that shows the customer's history
          of tiers (e.g. this customer started out on the Pro Plan, then downgraded to
          the Starter plan).

        ### Usage guidelines:

        Use the `starting_at`, `covering_date`, and `include_archived` parameters to
        filter the list of returned contracts. For example, to list only currently
        active contracts, pass `covering_date` equal to the current time.

        Args:
          covering_date: Optional RFC 3339 timestamp. Only include contracts active on the provided date.
              This cannot be provided if starting_at filter is provided.

          include_archived: Include archived contracts in the response.

          include_balance: Include the balance of credits and commits in the response. Setting this flag
              may cause the response to be slower.

          include_ledgers: Include commit/credit ledgers in the response. Setting this flag may cause the
              response to be slower.

          starting_at: Optional RFC 3339 timestamp. Only include contracts that started on or after
              this date. This cannot be provided if covering_date filter is provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/contracts/list",
            body=await async_maybe_transform(
                {
                    "customer_id": customer_id,
                    "covering_date": covering_date,
                    "include_archived": include_archived,
                    "include_balance": include_balance,
                    "include_ledgers": include_ledgers,
                    "starting_at": starting_at,
                },
                contract_list_params.ContractListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractListResponse,
        )

    async def edit(
        self,
        *,
        contract_id: str,
        customer_id: str,
        add_billing_provider_configuration_update: contract_edit_params.AddBillingProviderConfigurationUpdate
        | Omit = omit,
        add_commits: Iterable[contract_edit_params.AddCommit] | Omit = omit,
        add_credits: Iterable[contract_edit_params.AddCredit] | Omit = omit,
        add_discounts: Iterable[contract_edit_params.AddDiscount] | Omit = omit,
        add_overrides: Iterable[contract_edit_params.AddOverride] | Omit = omit,
        add_prepaid_balance_threshold_configuration: PrepaidBalanceThresholdConfigurationV2 | Omit = omit,
        add_professional_services: Iterable[contract_edit_params.AddProfessionalService] | Omit = omit,
        add_recurring_commits: Iterable[contract_edit_params.AddRecurringCommit] | Omit = omit,
        add_recurring_credits: Iterable[contract_edit_params.AddRecurringCredit] | Omit = omit,
        add_reseller_royalties: Iterable[contract_edit_params.AddResellerRoyalty] | Omit = omit,
        add_scheduled_charges: Iterable[contract_edit_params.AddScheduledCharge] | Omit = omit,
        add_spend_threshold_configuration: SpendThresholdConfigurationV2 | Omit = omit,
        add_subscriptions: Iterable[contract_edit_params.AddSubscription] | Omit = omit,
        allow_contract_ending_before_finalized_invoice: bool | Omit = omit,
        archive_commits: Iterable[contract_edit_params.ArchiveCommit] | Omit = omit,
        archive_credits: Iterable[contract_edit_params.ArchiveCredit] | Omit = omit,
        archive_scheduled_charges: Iterable[contract_edit_params.ArchiveScheduledCharge] | Omit = omit,
        remove_overrides: Iterable[contract_edit_params.RemoveOverride] | Omit = omit,
        uniqueness_key: str | Omit = omit,
        update_commits: Iterable[contract_edit_params.UpdateCommit] | Omit = omit,
        update_contract_end_date: Union[str, datetime, None] | Omit = omit,
        update_contract_name: Optional[str] | Omit = omit,
        update_credits: Iterable[contract_edit_params.UpdateCredit] | Omit = omit,
        update_prepaid_balance_threshold_configuration: contract_edit_params.UpdatePrepaidBalanceThresholdConfiguration
        | Omit = omit,
        update_recurring_commits: Iterable[contract_edit_params.UpdateRecurringCommit] | Omit = omit,
        update_recurring_credits: Iterable[contract_edit_params.UpdateRecurringCredit] | Omit = omit,
        update_scheduled_charges: Iterable[contract_edit_params.UpdateScheduledCharge] | Omit = omit,
        update_spend_threshold_configuration: contract_edit_params.UpdateSpendThresholdConfiguration | Omit = omit,
        update_subscriptions: Iterable[contract_edit_params.UpdateSubscription] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractEditResponse:
        """
        The ability to edit a contract helps you react quickly to the needs of your
        customers and your business.

        ### Use this endpoint to:

        - Encode mid-term commitment and discount changes
        - Fix configuration mistakes and easily roll back packaging changes

        ### Key response fields:

        - The `id` of the edit
        - Complete edit details. For example, if you edited the contract to add new
          overrides and credits, you will receive the IDs of those overrides and credits
          in the response.

        ### Usage guidelines:

        - When you edit a contract, any draft invoices update immediately to reflect
          that edit. Finalized invoices remain unchanged - you must void and regenerate
          them in the UI or API to reflect the edit.
        - Contract editing must be enabled to use this endpoint. Reach out to your
          Metronome representative to learn more.

        Args:
          contract_id: ID of the contract being edited

          customer_id: ID of the customer whose contract is being edited

          add_billing_provider_configuration_update: Update the billing provider configuration on the contract. Currently only
              supports adding a billing provider configuration to a contract that does not
              already have one.

          add_professional_services: This field's availability is dependent on your client's configuration.

          add_subscriptions: Optional list of
              [subscriptions](https://docs.metronome.com/manage-product-access/create-subscription/)
              to add to the contract.

          allow_contract_ending_before_finalized_invoice: If true, allows setting the contract end date earlier than the end_timestamp of
              existing finalized invoices. Finalized invoices will be unchanged; if you want
              to incorporate the new end date, you can void and regenerate finalized usage
              invoices. Defaults to true.

          archive_commits: IDs of commits to archive

          archive_credits: IDs of credits to archive

          archive_scheduled_charges: IDs of scheduled charges to archive

          remove_overrides: IDs of overrides to remove

          uniqueness_key: Optional uniqueness key to prevent duplicate contract edits.

          update_contract_end_date: RFC 3339 timestamp indicating when the contract will end (exclusive).

          update_contract_name: Value to update the contract name to. If not provided, the contract name will
              remain unchanged.

          update_recurring_commits: Edits to these recurring commits will only affect commits whose access schedules
              has not started. Expired commits, and commits with an active access schedule
              will remain unchanged.

          update_recurring_credits: Edits to these recurring credits will only affect credits whose access schedules
              has not started. Expired credits, and credits with an active access schedule
              will remain unchanged.

          update_subscriptions: Optional list of subscriptions to update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/contracts/edit",
            body=await async_maybe_transform(
                {
                    "contract_id": contract_id,
                    "customer_id": customer_id,
                    "add_billing_provider_configuration_update": add_billing_provider_configuration_update,
                    "add_commits": add_commits,
                    "add_credits": add_credits,
                    "add_discounts": add_discounts,
                    "add_overrides": add_overrides,
                    "add_prepaid_balance_threshold_configuration": add_prepaid_balance_threshold_configuration,
                    "add_professional_services": add_professional_services,
                    "add_recurring_commits": add_recurring_commits,
                    "add_recurring_credits": add_recurring_credits,
                    "add_reseller_royalties": add_reseller_royalties,
                    "add_scheduled_charges": add_scheduled_charges,
                    "add_spend_threshold_configuration": add_spend_threshold_configuration,
                    "add_subscriptions": add_subscriptions,
                    "allow_contract_ending_before_finalized_invoice": allow_contract_ending_before_finalized_invoice,
                    "archive_commits": archive_commits,
                    "archive_credits": archive_credits,
                    "archive_scheduled_charges": archive_scheduled_charges,
                    "remove_overrides": remove_overrides,
                    "uniqueness_key": uniqueness_key,
                    "update_commits": update_commits,
                    "update_contract_end_date": update_contract_end_date,
                    "update_contract_name": update_contract_name,
                    "update_credits": update_credits,
                    "update_prepaid_balance_threshold_configuration": update_prepaid_balance_threshold_configuration,
                    "update_recurring_commits": update_recurring_commits,
                    "update_recurring_credits": update_recurring_credits,
                    "update_scheduled_charges": update_scheduled_charges,
                    "update_spend_threshold_configuration": update_spend_threshold_configuration,
                    "update_subscriptions": update_subscriptions,
                },
                contract_edit_params.ContractEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractEditResponse,
        )

    async def edit_commit(
        self,
        *,
        commit_id: str,
        customer_id: str,
        access_schedule: contract_edit_commit_params.AccessSchedule | Omit = omit,
        applicable_product_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        applicable_product_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        description: str | Omit = omit,
        hierarchy_configuration: CommitHierarchyConfiguration | Omit = omit,
        invoice_contract_id: str | Omit = omit,
        invoice_schedule: contract_edit_commit_params.InvoiceSchedule | Omit = omit,
        name: str | Omit = omit,
        priority: Optional[float] | Omit = omit,
        product_id: str | Omit = omit,
        rate_type: Literal["LIST_RATE", "COMMIT_RATE"] | Omit = omit,
        specifiers: Optional[Iterable[CommitSpecifierInput]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractEditCommitResponse:
        """Edit specific details for a contract-level or customer-level commit.

        Use this
        endpoint to modify individual commit access schedules, invoice schedules,
        applicable products, invoicing contracts, or other fields.

        ### Usage guidelines:

        - As with all edits in Metronome, draft invoices will reflect the edit
          immediately, while finalized invoices are untouched unless voided and
          regenerated.
        - If a commit's invoice schedule item is associated with a finalized invoice,
          you cannot remove or update the invoice schedule item.
        - If a commit's invoice schedule item is associated with a voided invoice, you
          cannot remove the invoice schedule item.
        - You cannot remove an commit access schedule segment that was applied to a
          finalized invoice. You can void the invoice beforehand and then remove the
          access schedule segment.

        Args:
          commit_id: ID of the commit to edit

          customer_id: ID of the customer whose commit is being edited

          applicable_product_ids: Which products the commit applies to. If applicable_product_ids,
              applicable_product_tags or specifiers are not provided, the commit applies to
              all products.

          applicable_product_tags: Which tags the commit applies to. If applicable_product_ids,
              applicable_product_tags or specifiers are not provided, the commit applies to
              all products.

          description: Updated description for the commit

          hierarchy_configuration: Optional configuration for commit hierarchy access control

          invoice_contract_id: ID of contract to use for invoicing

          name: Updated name for the commit

          priority: If multiple commits are applicable, the one with the lower priority will apply
              first.

          rate_type: If provided, updates the commit to use the specified rate type for current and
              future invoices. Previously finalized invoices will need to be voided and
              regenerated to reflect the rate type change.

          specifiers: List of filters that determine what kind of customer usage draws down a commit
              or credit. A customer's usage needs to meet the condition of at least one of the
              specifiers to contribute to a commit's or credit's drawdown. This field cannot
              be used together with `applicable_product_ids` or `applicable_product_tags`.
              Instead, to target usage by product or product tag, pass those values in the
              body of `specifiers`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/contracts/commits/edit",
            body=await async_maybe_transform(
                {
                    "commit_id": commit_id,
                    "customer_id": customer_id,
                    "access_schedule": access_schedule,
                    "applicable_product_ids": applicable_product_ids,
                    "applicable_product_tags": applicable_product_tags,
                    "description": description,
                    "hierarchy_configuration": hierarchy_configuration,
                    "invoice_contract_id": invoice_contract_id,
                    "invoice_schedule": invoice_schedule,
                    "name": name,
                    "priority": priority,
                    "product_id": product_id,
                    "rate_type": rate_type,
                    "specifiers": specifiers,
                },
                contract_edit_commit_params.ContractEditCommitParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractEditCommitResponse,
        )

    async def edit_credit(
        self,
        *,
        credit_id: str,
        customer_id: str,
        access_schedule: contract_edit_credit_params.AccessSchedule | Omit = omit,
        applicable_product_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        applicable_product_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        description: str | Omit = omit,
        hierarchy_configuration: CommitHierarchyConfiguration | Omit = omit,
        name: str | Omit = omit,
        priority: Optional[float] | Omit = omit,
        product_id: str | Omit = omit,
        rate_type: Literal["LIST_RATE", "COMMIT_RATE"] | Omit = omit,
        specifiers: Optional[Iterable[CommitSpecifierInput]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractEditCreditResponse:
        """
        Edit details for a contract-level or customer-level credit.

        ### Use this endpoint to:

        - Extend the duration or the amount of an existing free credit like a trial
        - Modify individual credit access schedules, applicable products, priority, or
          other fields.

        ### Usage guidelines:

        - As with all edits in Metronome, draft invoices will reflect the edit
          immediately, while finalized invoices are untouched unless voided and
          regenerated.
        - You cannot remove an access schedule segment that was applied to a finalized
          invoice. You can void the invoice beforehand and then remove the access
          schedule segment.

        Args:
          credit_id: ID of the credit to edit

          customer_id: ID of the customer whose credit is being edited

          applicable_product_ids: Which products the credit applies to. If both applicable_product_ids and
              applicable_product_tags are not provided, the credit applies to all products.

          applicable_product_tags: Which tags the credit applies to. If both applicable_product_ids and
              applicable_product_tags are not provided, the credit applies to all products.

          description: Updated description for the credit

          hierarchy_configuration: Optional configuration for credit hierarchy access control

          name: Updated name for the credit

          priority: If multiple commits are applicable, the one with the lower priority will apply
              first.

          rate_type: If provided, updates the credit to use the specified rate type for current and
              future invoices. Previously finalized invoices will need to be voided and
              regenerated to reflect the rate type change.

          specifiers: List of filters that determine what kind of customer usage draws down a commit
              or credit. A customer's usage needs to meet the condition of at least one of the
              specifiers to contribute to a commit's or credit's drawdown. This field cannot
              be used together with `applicable_product_ids` or `applicable_product_tags`.
              Instead, to target usage by product or product tag, pass those values in the
              body of `specifiers`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/contracts/credits/edit",
            body=await async_maybe_transform(
                {
                    "credit_id": credit_id,
                    "customer_id": customer_id,
                    "access_schedule": access_schedule,
                    "applicable_product_ids": applicable_product_ids,
                    "applicable_product_tags": applicable_product_tags,
                    "description": description,
                    "hierarchy_configuration": hierarchy_configuration,
                    "name": name,
                    "priority": priority,
                    "product_id": product_id,
                    "rate_type": rate_type,
                    "specifiers": specifiers,
                },
                contract_edit_credit_params.ContractEditCreditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractEditCreditResponse,
        )

    async def get_edit_history(
        self,
        *,
        contract_id: str,
        customer_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContractGetEditHistoryResponse:
        """List all the edits made to a contract over time.

        In Metronome, you can edit a
        contract at any point after it's created to fix mistakes or reflect changes in
        terms. Metronome stores a full history of all edits that were ever made to a
        contract, whether through the UI, `editContract` endpoint, or other endpoints
        like `updateContractEndDate`.

        ### Use this endpoint to:

        - Understand what changes were made to a contract, when, and by who

        ### Key response fields:

        - An array of every edit ever made to the contract
        - Details on each individual edit - for example showing that in one edit, a user
          added two discounts and incremented a subscription quantity.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/contracts/getEditHistory",
            body=await async_maybe_transform(
                {
                    "contract_id": contract_id,
                    "customer_id": customer_id,
                },
                contract_get_edit_history_params.ContractGetEditHistoryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractGetEditHistoryResponse,
        )


class ContractsResourceWithRawResponse:
    def __init__(self, contracts: ContractsResource) -> None:
        self._contracts = contracts

        self.retrieve = to_raw_response_wrapper(
            contracts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            contracts.list,
        )
        self.edit = to_raw_response_wrapper(
            contracts.edit,
        )
        self.edit_commit = to_raw_response_wrapper(
            contracts.edit_commit,
        )
        self.edit_credit = to_raw_response_wrapper(
            contracts.edit_credit,
        )
        self.get_edit_history = to_raw_response_wrapper(
            contracts.get_edit_history,
        )


class AsyncContractsResourceWithRawResponse:
    def __init__(self, contracts: AsyncContractsResource) -> None:
        self._contracts = contracts

        self.retrieve = async_to_raw_response_wrapper(
            contracts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            contracts.list,
        )
        self.edit = async_to_raw_response_wrapper(
            contracts.edit,
        )
        self.edit_commit = async_to_raw_response_wrapper(
            contracts.edit_commit,
        )
        self.edit_credit = async_to_raw_response_wrapper(
            contracts.edit_credit,
        )
        self.get_edit_history = async_to_raw_response_wrapper(
            contracts.get_edit_history,
        )


class ContractsResourceWithStreamingResponse:
    def __init__(self, contracts: ContractsResource) -> None:
        self._contracts = contracts

        self.retrieve = to_streamed_response_wrapper(
            contracts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            contracts.list,
        )
        self.edit = to_streamed_response_wrapper(
            contracts.edit,
        )
        self.edit_commit = to_streamed_response_wrapper(
            contracts.edit_commit,
        )
        self.edit_credit = to_streamed_response_wrapper(
            contracts.edit_credit,
        )
        self.get_edit_history = to_streamed_response_wrapper(
            contracts.get_edit_history,
        )


class AsyncContractsResourceWithStreamingResponse:
    def __init__(self, contracts: AsyncContractsResource) -> None:
        self._contracts = contracts

        self.retrieve = async_to_streamed_response_wrapper(
            contracts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            contracts.list,
        )
        self.edit = async_to_streamed_response_wrapper(
            contracts.edit,
        )
        self.edit_commit = async_to_streamed_response_wrapper(
            contracts.edit_commit,
        )
        self.edit_credit = async_to_streamed_response_wrapper(
            contracts.edit_credit,
        )
        self.get_edit_history = async_to_streamed_response_wrapper(
            contracts.get_edit_history,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1 import (
    PlanListResponse,
    PlanGetDetailsResponse,
    PlanListChargesResponse,
    PlanListCustomersResponse,
)
from metronome.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPlans:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: Metronome) -> None:
        plan = client.v1.plans.list()
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Metronome) -> None:
        plan = client.v1.plans.list(
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Metronome) -> None:
        response = client.v1.plans.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Metronome) -> None:
        with client.v1.plans.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get_details(self, client: Metronome) -> None:
        plan = client.v1.plans.get_details(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(PlanGetDetailsResponse, plan, path=["response"])

    @parametrize
    def test_raw_response_get_details(self, client: Metronome) -> None:
        response = client.v1.plans.with_raw_response.get_details(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanGetDetailsResponse, plan, path=["response"])

    @parametrize
    def test_streaming_response_get_details(self, client: Metronome) -> None:
        with client.v1.plans.with_streaming_response.get_details(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanGetDetailsResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_get_details(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            client.v1.plans.with_raw_response.get_details(
                plan_id="",
            )

    @parametrize
    def test_method_list_charges(self, client: Metronome) -> None:
        plan = client.v1.plans.list_charges(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(SyncCursorPage[PlanListChargesResponse], plan, path=["response"])

    @parametrize
    def test_method_list_charges_with_all_params(self, client: Metronome) -> None:
        plan = client.v1.plans.list_charges(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(SyncCursorPage[PlanListChargesResponse], plan, path=["response"])

    @parametrize
    def test_raw_response_list_charges(self, client: Metronome) -> None:
        response = client.v1.plans.with_raw_response.list_charges(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(SyncCursorPage[PlanListChargesResponse], plan, path=["response"])

    @parametrize
    def test_streaming_response_list_charges(self, client: Metronome) -> None:
        with client.v1.plans.with_streaming_response.list_charges(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(SyncCursorPage[PlanListChargesResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_charges(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            client.v1.plans.with_raw_response.list_charges(
                plan_id="",
            )

    @parametrize
    def test_method_list_customers(self, client: Metronome) -> None:
        plan = client.v1.plans.list_customers(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(SyncCursorPage[PlanListCustomersResponse], plan, path=["response"])

    @parametrize
    def test_method_list_customers_with_all_params(self, client: Metronome) -> None:
        plan = client.v1.plans.list_customers(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            limit=1,
            next_page="next_page",
            status="all",
        )
        assert_matches_type(SyncCursorPage[PlanListCustomersResponse], plan, path=["response"])

    @parametrize
    def test_raw_response_list_customers(self, client: Metronome) -> None:
        response = client.v1.plans.with_raw_response.list_customers(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(SyncCursorPage[PlanListCustomersResponse], plan, path=["response"])

    @parametrize
    def test_streaming_response_list_customers(self, client: Metronome) -> None:
        with client.v1.plans.with_streaming_response.list_customers(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(SyncCursorPage[PlanListCustomersResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_customers(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            client.v1.plans.with_raw_response.list_customers(
                plan_id="",
            )


class TestAsyncPlans:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_list(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.plans.list()
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.plans.list(
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.plans.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.plans.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get_details(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.plans.get_details(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(PlanGetDetailsResponse, plan, path=["response"])

    @parametrize
    async def test_raw_response_get_details(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.plans.with_raw_response.get_details(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanGetDetailsResponse, plan, path=["response"])

    @parametrize
    async def test_streaming_response_get_details(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.plans.with_streaming_response.get_details(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanGetDetailsResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_get_details(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            await async_client.v1.plans.with_raw_response.get_details(
                plan_id="",
            )

    @parametrize
    async def test_method_list_charges(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.plans.list_charges(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(AsyncCursorPage[PlanListChargesResponse], plan, path=["response"])

    @parametrize
    async def test_method_list_charges_with_all_params(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.plans.list_charges(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(AsyncCursorPage[PlanListChargesResponse], plan, path=["response"])

    @parametrize
    async def test_raw_response_list_charges(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.plans.with_raw_response.list_charges(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(AsyncCursorPage[PlanListChargesResponse], plan, path=["response"])

    @parametrize
    async def test_streaming_response_list_charges(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.plans.with_streaming_response.list_charges(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(AsyncCursorPage[PlanListChargesResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_charges(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            await async_client.v1.plans.with_raw_response.list_charges(
                plan_id="",
            )

    @parametrize
    async def test_method_list_customers(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.plans.list_customers(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )
        assert_matches_type(AsyncCursorPage[PlanListCustomersResponse], plan, path=["response"])

    @parametrize
    async def test_method_list_customers_with_all_params(self, async_client: AsyncMetronome) -> None:
        plan = await async_client.v1.plans.list_customers(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            limit=1,
            next_page="next_page",
            status="all",
        )
        assert_matches_type(AsyncCursorPage[PlanListCustomersResponse], plan, path=["response"])

    @parametrize
    async def test_raw_response_list_customers(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.plans.with_raw_response.list_customers(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(AsyncCursorPage[PlanListCustomersResponse], plan, path=["response"])

    @parametrize
    async def test_streaming_response_list_customers(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.plans.with_streaming_response.list_customers(
            plan_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(AsyncCursorPage[PlanListCustomersResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_customers(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            await async_client.v1.plans.with_raw_response.list_customers(
                plan_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1 import (
    CustomFieldListKeysResponse,
)
from metronome.pagination import SyncCursorPageWithoutLimit, AsyncCursorPageWithoutLimit

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCustomFields:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_add_key(self, client: Metronome) -> None:
        custom_field = client.v1.custom_fields.add_key(
            enforce_uniqueness=True,
            entity="customer",
            key="x_account_id",
        )
        assert custom_field is None

    @parametrize
    def test_raw_response_add_key(self, client: Metronome) -> None:
        response = client.v1.custom_fields.with_raw_response.add_key(
            enforce_uniqueness=True,
            entity="customer",
            key="x_account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = response.parse()
        assert custom_field is None

    @parametrize
    def test_streaming_response_add_key(self, client: Metronome) -> None:
        with client.v1.custom_fields.with_streaming_response.add_key(
            enforce_uniqueness=True,
            entity="customer",
            key="x_account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = response.parse()
            assert custom_field is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete_values(self, client: Metronome) -> None:
        custom_field = client.v1.custom_fields.delete_values(
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
            keys=["x_account_id"],
        )
        assert custom_field is None

    @parametrize
    def test_raw_response_delete_values(self, client: Metronome) -> None:
        response = client.v1.custom_fields.with_raw_response.delete_values(
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
            keys=["x_account_id"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = response.parse()
        assert custom_field is None

    @parametrize
    def test_streaming_response_delete_values(self, client: Metronome) -> None:
        with client.v1.custom_fields.with_streaming_response.delete_values(
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
            keys=["x_account_id"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = response.parse()
            assert custom_field is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list_keys(self, client: Metronome) -> None:
        custom_field = client.v1.custom_fields.list_keys()
        assert_matches_type(SyncCursorPageWithoutLimit[CustomFieldListKeysResponse], custom_field, path=["response"])

    @parametrize
    def test_method_list_keys_with_all_params(self, client: Metronome) -> None:
        custom_field = client.v1.custom_fields.list_keys(
            next_page="next_page",
            entities=["customer"],
        )
        assert_matches_type(SyncCursorPageWithoutLimit[CustomFieldListKeysResponse], custom_field, path=["response"])

    @parametrize
    def test_raw_response_list_keys(self, client: Metronome) -> None:
        response = client.v1.custom_fields.with_raw_response.list_keys()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = response.parse()
        assert_matches_type(SyncCursorPageWithoutLimit[CustomFieldListKeysResponse], custom_field, path=["response"])

    @parametrize
    def test_streaming_response_list_keys(self, client: Metronome) -> None:
        with client.v1.custom_fields.with_streaming_response.list_keys() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = response.parse()
            assert_matches_type(
                SyncCursorPageWithoutLimit[CustomFieldListKeysResponse], custom_field, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_remove_key(self, client: Metronome) -> None:
        custom_field = client.v1.custom_fields.remove_key(
            entity="customer",
            key="x_account_id",
        )
        assert custom_field is None

    @parametrize
    def test_raw_response_remove_key(self, client: Metronome) -> None:
        response = client.v1.custom_fields.with_raw_response.remove_key(
            entity="customer",
            key="x_account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = response.parse()
        assert custom_field is None

    @parametrize
    def test_streaming_response_remove_key(self, client: Metronome) -> None:
        with client.v1.custom_fields.with_streaming_response.remove_key(
            entity="customer",
            key="x_account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = response.parse()
            assert custom_field is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_set_values(self, client: Metronome) -> None:
        custom_field = client.v1.custom_fields.set_values(
            custom_fields={"x_account_id": "KyVnHhSBWl7eY2bl"},
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
        )
        assert custom_field is None

    @parametrize
    def test_raw_response_set_values(self, client: Metronome) -> None:
        response = client.v1.custom_fields.with_raw_response.set_values(
            custom_fields={"x_account_id": "KyVnHhSBWl7eY2bl"},
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = response.parse()
        assert custom_field is None

    @parametrize
    def test_streaming_response_set_values(self, client: Metronome) -> None:
        with client.v1.custom_fields.with_streaming_response.set_values(
            custom_fields={"x_account_id": "KyVnHhSBWl7eY2bl"},
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = response.parse()
            assert custom_field is None

        assert cast(Any, response.is_closed) is True


class TestAsyncCustomFields:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_add_key(self, async_client: AsyncMetronome) -> None:
        custom_field = await async_client.v1.custom_fields.add_key(
            enforce_uniqueness=True,
            entity="customer",
            key="x_account_id",
        )
        assert custom_field is None

    @parametrize
    async def test_raw_response_add_key(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.custom_fields.with_raw_response.add_key(
            enforce_uniqueness=True,
            entity="customer",
            key="x_account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = await response.parse()
        assert custom_field is None

    @parametrize
    async def test_streaming_response_add_key(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.custom_fields.with_streaming_response.add_key(
            enforce_uniqueness=True,
            entity="customer",
            key="x_account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = await response.parse()
            assert custom_field is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete_values(self, async_client: AsyncMetronome) -> None:
        custom_field = await async_client.v1.custom_fields.delete_values(
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
            keys=["x_account_id"],
        )
        assert custom_field is None

    @parametrize
    async def test_raw_response_delete_values(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.custom_fields.with_raw_response.delete_values(
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
            keys=["x_account_id"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = await response.parse()
        assert custom_field is None

    @parametrize
    async def test_streaming_response_delete_values(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.custom_fields.with_streaming_response.delete_values(
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
            keys=["x_account_id"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = await response.parse()
            assert custom_field is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list_keys(self, async_client: AsyncMetronome) -> None:
        custom_field = await async_client.v1.custom_fields.list_keys()
        assert_matches_type(AsyncCursorPageWithoutLimit[CustomFieldListKeysResponse], custom_field, path=["response"])

    @parametrize
    async def test_method_list_keys_with_all_params(self, async_client: AsyncMetronome) -> None:
        custom_field = await async_client.v1.custom_fields.list_keys(
            next_page="next_page",
            entities=["customer"],
        )
        assert_matches_type(AsyncCursorPageWithoutLimit[CustomFieldListKeysResponse], custom_field, path=["response"])

    @parametrize
    async def test_raw_response_list_keys(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.custom_fields.with_raw_response.list_keys()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = await response.parse()
        assert_matches_type(AsyncCursorPageWithoutLimit[CustomFieldListKeysResponse], custom_field, path=["response"])

    @parametrize
    async def test_streaming_response_list_keys(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.custom_fields.with_streaming_response.list_keys() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = await response.parse()
            assert_matches_type(
                AsyncCursorPageWithoutLimit[CustomFieldListKeysResponse], custom_field, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_remove_key(self, async_client: AsyncMetronome) -> None:
        custom_field = await async_client.v1.custom_fields.remove_key(
            entity="customer",
            key="x_account_id",
        )
        assert custom_field is None

    @parametrize
    async def test_raw_response_remove_key(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.custom_fields.with_raw_response.remove_key(
            entity="customer",
            key="x_account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = await response.parse()
        assert custom_field is None

    @parametrize
    async def test_streaming_response_remove_key(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.custom_fields.with_streaming_response.remove_key(
            entity="customer",
            key="x_account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = await response.parse()
            assert custom_field is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_set_values(self, async_client: AsyncMetronome) -> None:
        custom_field = await async_client.v1.custom_fields.set_values(
            custom_fields={"x_account_id": "KyVnHhSBWl7eY2bl"},
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
        )
        assert custom_field is None

    @parametrize
    async def test_raw_response_set_values(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.custom_fields.with_raw_response.set_values(
            custom_fields={"x_account_id": "KyVnHhSBWl7eY2bl"},
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        custom_field = await response.parse()
        assert custom_field is None

    @parametrize
    async def test_streaming_response_set_values(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.custom_fields.with_streaming_response.set_values(
            custom_fields={"x_account_id": "KyVnHhSBWl7eY2bl"},
            entity="customer",
            entity_id="99594816-e8a5-4bca-be21-8d1de0f45120",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            custom_field = await response.parse()
            assert custom_field is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RolloverAmountMaxAmountParam"]


class RolloverAmountMaxAmountParam(TypedDict, total=False):
    type: Required[Literal["MAX_AMOUNT"]]
    """Rollover up to a fixed amount of the original credit grant amount."""

    value: Required[float]
    """The maximum amount to rollover."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorPageWithoutLimit, AsyncCursorPageWithoutLimit
from ...._base_client import AsyncPaginator, make_request_options
from ....types.v1.customers import alert_list_params, alert_reset_params, alert_retrieve_params
from ....types.v1.customers.customer_alert import CustomerAlert
from ....types.v1.customers.alert_retrieve_response import AlertRetrieveResponse

__all__ = ["AlertsResource", "AsyncAlertsResource"]


class AlertsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AlertsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AlertsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AlertsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AlertsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        alert_id: str,
        customer_id: str,
        group_values: Iterable[alert_retrieve_params.GroupValue] | Omit = omit,
        plans_or_contracts: Literal["PLANS", "CONTRACTS"] | Omit = omit,
        seat_filter: alert_retrieve_params.SeatFilter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlertRetrieveResponse:
        """
        Retrieve the real-time evaluation status for a specific threshold
        notification-customer pair. This endpoint provides instant visibility into
        whether a customer has triggered a threshold notification condition, enabling
        you to monitor account health and take proactive action based on current
        threshold notification states.

        ### Use this endpoint to:

        - Check if a specific customer is currently violating an threshold notification
          (`in_alarm` status)
        - Verify threshold notification configuration details and threshold values for a
          customer
        - Monitor the evaluation state of newly created or recently modified threshold
          notification
        - Build dashboards or automated workflows that respond to specific threshold
          notification conditions
        - Validate threshold notification behavior before deploying to production
          customers
        - Integrate threshold notification status checks into customer support tools or
          admin interfaces

        ### Key response fields:

        A CustomerAlert object containing:

        - `customer_status`: The current evaluation state

        - `ok` - Customer is within acceptable thresholds
        - `in_alarm` - Customer has breached the threshold for the notification
        - `evaluating` - Notification is currently being evaluated (typically during
          initial setup)
        - `null` - Notification has been archived
        - `triggered_by`: Additional context about what caused the notification to
          trigger (when applicable)
        - alert: Complete threshold notification configuration including:
          - Notification ID, name, and type
          - Current threshold values and credit type information
          - Notification status (enabled, disabled, or archived)
          - Last update timestamp
          - Any applied filters (credit grant types, custom fields, group values)

        ### Usage guidelines:

        - Customer status: Returns the current evaluation state, not historical data.
          For threshold notification history, use webhook notifications or event logs
        - Required parameters: Both customer_id and alert_id must be valid UUIDs that
          exist in your organization
        - Archived notifications: Returns null for customer_status if the notification
          has been archived, but still includes the notification configuration details
        - Performance considerations: This endpoint queries live evaluation state,
          making it ideal for real-time monitoring but not for bulk status checks
        - Integration patterns: Best used for on-demand status checks in response to
          user actions or as part of targeted monitoring workflows
        - Error handling: Returns 404 if either the customer or alert_id doesn't exist
          or isn't accessible to your organization

        Args:
          alert_id: The Metronome ID of the threshold notification

          customer_id: The Metronome ID of the customer

          group_values: Only present for `spend_threshold_reached` notifications. Retrieve the
              notification for a specific group key-value pair.

          plans_or_contracts: When parallel threshold notifications are enabled during migration, this flag
              denotes whether to fetch notifications for plans or contracts.

          seat_filter: Only allowed for `low_remaining_seat_balance_reached` notifications. This
              filters alerts by the seat group key-value pair.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/customer-alerts/get",
            body=maybe_transform(
                {
                    "alert_id": alert_id,
                    "customer_id": customer_id,
                    "group_values": group_values,
                    "plans_or_contracts": plans_or_contracts,
                    "seat_filter": seat_filter,
                },
                alert_retrieve_params.AlertRetrieveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertRetrieveResponse,
        )

    def list(
        self,
        *,
        customer_id: str,
        next_page: str | Omit = omit,
        alert_statuses: List[Literal["ENABLED", "DISABLED", "ARCHIVED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPageWithoutLimit[CustomerAlert]:
        """
        Retrieve all threshold notification configurations and their current statuses
        for a specific customer in a single API call. This endpoint provides a
        comprehensive view of all threshold notification monitoring a customer account.

        ### Use this endpoint to:

        - Display all active threshold notifications for a customer in dashboards or
          admin panels
        - Quickly identify which threshold notifications a customer is currently
          triggering
        - Audit threshold notification coverage for specific accounts
        - Filter threshold notifications by status (enabled, disabled, or archived)

        ### Key response fields:

        - data: Array of CustomerAlert objects, each containing:
          - Current evaluation status (`ok`, `in_alarm`, `evaluating`, or `null`)
          - Complete threshold notification configuration and threshold details
          - Threshold notification metadata including type, name, and last update time
        - next_page: Pagination cursor for retrieving additional results

        ### Usage guidelines:

        - Default behavior: Returns only enabled threshold notifications unless
          `alert_statuses` filter is specified
        - Pagination: Use the `next_page` cursor to retrieve all results for customers
          with many notifications
        - Performance: Efficiently retrieves multiple threshold notification statuses in
          a single request instead of making individual calls
        - Filtering: Pass the `alert_statuses` array to include disabled or archived
          threshold notifications in results

        Args:
          customer_id: The Metronome ID of the customer

          next_page: Cursor that indicates where the next page of results should start.

          alert_statuses: Optionally filter by threshold notification status. If absent, only enabled
              notifications will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/customer-alerts/list",
            page=SyncCursorPageWithoutLimit[CustomerAlert],
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "alert_statuses": alert_statuses,
                },
                alert_list_params.AlertListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"next_page": next_page}, alert_list_params.AlertListParams),
            ),
            model=CustomerAlert,
            method="post",
        )

    def reset(
        self,
        *,
        alert_id: str,
        customer_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Force an immediate re-evaluation of a specific threshold notification for a
        customer, clearing any previous state and triggering a fresh assessment against
        current thresholds. This endpoint ensures threshold notification accuracy after
        configuration changes or data corrections.

        ### Use this endpoint to:

        - Clear false positive threshold notifications after fixing data issues
        - Re-evaluate threshold notifications after adjusting customer balances or
          credits
        - Test threshold notification behavior during development and debugging
        - Resolve stuck threshold notification that may be in an incorrect state
        - Trigger immediate evaluation after threshold modifications

        ### Key response fields:

        - 200 Success: Confirmation that the threshold notification has been reset and
          re-evaluation initiated
        - No response body is returned - the operation completes asynchronously

        ### Usage guidelines:

        - Immediate effect: Triggers re-evaluation instantly, which may result in new
          webhook notifications if thresholds are breached
        - State clearing: Removes any cached evaluation state, ensuring a fresh
          assessment
        - Use sparingly: Intended for exceptional cases, not routine operations
        - Asynchronous processing: The reset completes immediately, but re-evaluation
          happens in the background

        Args:
          alert_id: The Metronome ID of the threshold notification

          customer_id: The Metronome ID of the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/v1/customer-alerts/reset",
            body=maybe_transform(
                {
                    "alert_id": alert_id,
                    "customer_id": customer_id,
                },
                alert_reset_params.AlertResetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncAlertsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAlertsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAlertsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAlertsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncAlertsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        alert_id: str,
        customer_id: str,
        group_values: Iterable[alert_retrieve_params.GroupValue] | Omit = omit,
        plans_or_contracts: Literal["PLANS", "CONTRACTS"] | Omit = omit,
        seat_filter: alert_retrieve_params.SeatFilter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AlertRetrieveResponse:
        """
        Retrieve the real-time evaluation status for a specific threshold
        notification-customer pair. This endpoint provides instant visibility into
        whether a customer has triggered a threshold notification condition, enabling
        you to monitor account health and take proactive action based on current
        threshold notification states.

        ### Use this endpoint to:

        - Check if a specific customer is currently violating an threshold notification
          (`in_alarm` status)
        - Verify threshold notification configuration details and threshold values for a
          customer
        - Monitor the evaluation state of newly created or recently modified threshold
          notification
        - Build dashboards or automated workflows that respond to specific threshold
          notification conditions
        - Validate threshold notification behavior before deploying to production
          customers
        - Integrate threshold notification status checks into customer support tools or
          admin interfaces

        ### Key response fields:

        A CustomerAlert object containing:

        - `customer_status`: The current evaluation state

        - `ok` - Customer is within acceptable thresholds
        - `in_alarm` - Customer has breached the threshold for the notification
        - `evaluating` - Notification is currently being evaluated (typically during
          initial setup)
        - `null` - Notification has been archived
        - `triggered_by`: Additional context about what caused the notification to
          trigger (when applicable)
        - alert: Complete threshold notification configuration including:
          - Notification ID, name, and type
          - Current threshold values and credit type information
          - Notification status (enabled, disabled, or archived)
          - Last update timestamp
          - Any applied filters (credit grant types, custom fields, group values)

        ### Usage guidelines:

        - Customer status: Returns the current evaluation state, not historical data.
          For threshold notification history, use webhook notifications or event logs
        - Required parameters: Both customer_id and alert_id must be valid UUIDs that
          exist in your organization
        - Archived notifications: Returns null for customer_status if the notification
          has been archived, but still includes the notification configuration details
        - Performance considerations: This endpoint queries live evaluation state,
          making it ideal for real-time monitoring but not for bulk status checks
        - Integration patterns: Best used for on-demand status checks in response to
          user actions or as part of targeted monitoring workflows
        - Error handling: Returns 404 if either the customer or alert_id doesn't exist
          or isn't accessible to your organization

        Args:
          alert_id: The Metronome ID of the threshold notification

          customer_id: The Metronome ID of the customer

          group_values: Only present for `spend_threshold_reached` notifications. Retrieve the
              notification for a specific group key-value pair.

          plans_or_contracts: When parallel threshold notifications are enabled during migration, this flag
              denotes whether to fetch notifications for plans or contracts.

          seat_filter: Only allowed for `low_remaining_seat_balance_reached` notifications. This
              filters alerts by the seat group key-value pair.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/customer-alerts/get",
            body=await async_maybe_transform(
                {
                    "alert_id": alert_id,
                    "customer_id": customer_id,
                    "group_values": group_values,
                    "plans_or_contracts": plans_or_contracts,
                    "seat_filter": seat_filter,
                },
                alert_retrieve_params.AlertRetrieveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AlertRetrieveResponse,
        )

    def list(
        self,
        *,
        customer_id: str,
        next_page: str | Omit = omit,
        alert_statuses: List[Literal["ENABLED", "DISABLED", "ARCHIVED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomerAlert, AsyncCursorPageWithoutLimit[CustomerAlert]]:
        """
        Retrieve all threshold notification configurations and their current statuses
        for a specific customer in a single API call. This endpoint provides a
        comprehensive view of all threshold notification monitoring a customer account.

        ### Use this endpoint to:

        - Display all active threshold notifications for a customer in dashboards or
          admin panels
        - Quickly identify which threshold notifications a customer is currently
          triggering
        - Audit threshold notification coverage for specific accounts
        - Filter threshold notifications by status (enabled, disabled, or archived)

        ### Key response fields:

        - data: Array of CustomerAlert objects, each containing:
          - Current evaluation status (`ok`, `in_alarm`, `evaluating`, or `null`)
          - Complete threshold notification configuration and threshold details
          - Threshold notification metadata including type, name, and last update time
        - next_page: Pagination cursor for retrieving additional results

        ### Usage guidelines:

        - Default behavior: Returns only enabled threshold notifications unless
          `alert_statuses` filter is specified
        - Pagination: Use the `next_page` cursor to retrieve all results for customers
          with many notifications
        - Performance: Efficiently retrieves multiple threshold notification statuses in
          a single request instead of making individual calls
        - Filtering: Pass the `alert_statuses` array to include disabled or archived
          threshold notifications in results

        Args:
          customer_id: The Metronome ID of the customer

          next_page: Cursor that indicates where the next page of results should start.

          alert_statuses: Optionally filter by threshold notification status. If absent, only enabled
              notifications will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/customer-alerts/list",
            page=AsyncCursorPageWithoutLimit[CustomerAlert],
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "alert_statuses": alert_statuses,
                },
                alert_list_params.AlertListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"next_page": next_page}, alert_list_params.AlertListParams),
            ),
            model=CustomerAlert,
            method="post",
        )

    async def reset(
        self,
        *,
        alert_id: str,
        customer_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Force an immediate re-evaluation of a specific threshold notification for a
        customer, clearing any previous state and triggering a fresh assessment against
        current thresholds. This endpoint ensures threshold notification accuracy after
        configuration changes or data corrections.

        ### Use this endpoint to:

        - Clear false positive threshold notifications after fixing data issues
        - Re-evaluate threshold notifications after adjusting customer balances or
          credits
        - Test threshold notification behavior during development and debugging
        - Resolve stuck threshold notification that may be in an incorrect state
        - Trigger immediate evaluation after threshold modifications

        ### Key response fields:

        - 200 Success: Confirmation that the threshold notification has been reset and
          re-evaluation initiated
        - No response body is returned - the operation completes asynchronously

        ### Usage guidelines:

        - Immediate effect: Triggers re-evaluation instantly, which may result in new
          webhook notifications if thresholds are breached
        - State clearing: Removes any cached evaluation state, ensuring a fresh
          assessment
        - Use sparingly: Intended for exceptional cases, not routine operations
        - Asynchronous processing: The reset completes immediately, but re-evaluation
          happens in the background

        Args:
          alert_id: The Metronome ID of the threshold notification

          customer_id: The Metronome ID of the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/v1/customer-alerts/reset",
            body=await async_maybe_transform(
                {
                    "alert_id": alert_id,
                    "customer_id": customer_id,
                },
                alert_reset_params.AlertResetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AlertsResourceWithRawResponse:
    def __init__(self, alerts: AlertsResource) -> None:
        self._alerts = alerts

        self.retrieve = to_raw_response_wrapper(
            alerts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            alerts.list,
        )
        self.reset = to_raw_response_wrapper(
            alerts.reset,
        )


class AsyncAlertsResourceWithRawResponse:
    def __init__(self, alerts: AsyncAlertsResource) -> None:
        self._alerts = alerts

        self.retrieve = async_to_raw_response_wrapper(
            alerts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            alerts.list,
        )
        self.reset = async_to_raw_response_wrapper(
            alerts.reset,
        )


class AlertsResourceWithStreamingResponse:
    def __init__(self, alerts: AlertsResource) -> None:
        self._alerts = alerts

        self.retrieve = to_streamed_response_wrapper(
            alerts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            alerts.list,
        )
        self.reset = to_streamed_response_wrapper(
            alerts.reset,
        )


class AsyncAlertsResourceWithStreamingResponse:
    def __init__(self, alerts: AsyncAlertsResource) -> None:
        self._alerts = alerts

        self.retrieve = async_to_streamed_response_wrapper(
            alerts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            alerts.list,
        )
        self.reset = async_to_streamed_response_wrapper(
            alerts.reset,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.contracts import named_schedule_update_params, named_schedule_retrieve_params
from ....types.v1.contracts.named_schedule_retrieve_response import NamedScheduleRetrieveResponse

__all__ = ["NamedSchedulesResource", "AsyncNamedSchedulesResource"]


class NamedSchedulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NamedSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return NamedSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NamedSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return NamedSchedulesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        rate_card_id: str,
        schedule_name: str,
        covering_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NamedScheduleRetrieveResponse:
        """Get a named schedule for the given rate card.

        This endpoint's availability is
        dependent on your client's configuration.

        Args:
          rate_card_id: ID of the rate card whose named schedule is to be retrieved

          schedule_name: The identifier for the schedule to be retrieved

          covering_date: If provided, at most one schedule segment will be returned (the one that covers
              this date). If not provided, all segments will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/getNamedSchedule",
            body=maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "schedule_name": schedule_name,
                    "covering_date": covering_date,
                },
                named_schedule_retrieve_params.NamedScheduleRetrieveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NamedScheduleRetrieveResponse,
        )

    def update(
        self,
        *,
        rate_card_id: str,
        schedule_name: str,
        starting_at: Union[str, datetime],
        value: object,
        ending_before: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Update a named schedule for the given rate card.

        This endpoint's availability is
        dependent on your client's configuration.

        Args:
          rate_card_id: ID of the rate card whose named schedule is to be updated

          schedule_name: The identifier for the schedule to be updated

          value: The value to set for the named schedule. The structure of this object is
              specific to the named schedule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/v1/contract-pricing/rate-cards/updateNamedSchedule",
            body=maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "schedule_name": schedule_name,
                    "starting_at": starting_at,
                    "value": value,
                    "ending_before": ending_before,
                },
                named_schedule_update_params.NamedScheduleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncNamedSchedulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNamedSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNamedSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNamedSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncNamedSchedulesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        rate_card_id: str,
        schedule_name: str,
        covering_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NamedScheduleRetrieveResponse:
        """Get a named schedule for the given rate card.

        This endpoint's availability is
        dependent on your client's configuration.

        Args:
          rate_card_id: ID of the rate card whose named schedule is to be retrieved

          schedule_name: The identifier for the schedule to be retrieved

          covering_date: If provided, at most one schedule segment will be returned (the one that covers
              this date). If not provided, all segments will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/getNamedSchedule",
            body=await async_maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "schedule_name": schedule_name,
                    "covering_date": covering_date,
                },
                named_schedule_retrieve_params.NamedScheduleRetrieveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NamedScheduleRetrieveResponse,
        )

    async def update(
        self,
        *,
        rate_card_id: str,
        schedule_name: str,
        starting_at: Union[str, datetime],
        value: object,
        ending_before: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Update a named schedule for the given rate card.

        This endpoint's availability is
        dependent on your client's configuration.

        Args:
          rate_card_id: ID of the rate card whose named schedule is to be updated

          schedule_name: The identifier for the schedule to be updated

          value: The value to set for the named schedule. The structure of this object is
              specific to the named schedule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/v1/contract-pricing/rate-cards/updateNamedSchedule",
            body=await async_maybe_transform(
                {
                    "rate_card_id": rate_card_id,
                    "schedule_name": schedule_name,
                    "starting_at": starting_at,
                    "value": value,
                    "ending_before": ending_before,
                },
                named_schedule_update_params.NamedScheduleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class NamedSchedulesResourceWithRawResponse:
    def __init__(self, named_schedules: NamedSchedulesResource) -> None:
        self._named_schedules = named_schedules

        self.retrieve = to_raw_response_wrapper(
            named_schedules.retrieve,
        )
        self.update = to_raw_response_wrapper(
            named_schedules.update,
        )


class AsyncNamedSchedulesResourceWithRawResponse:
    def __init__(self, named_schedules: AsyncNamedSchedulesResource) -> None:
        self._named_schedules = named_schedules

        self.retrieve = async_to_raw_response_wrapper(
            named_schedules.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            named_schedules.update,
        )


class NamedSchedulesResourceWithStreamingResponse:
    def __init__(self, named_schedules: NamedSchedulesResource) -> None:
        self._named_schedules = named_schedules

        self.retrieve = to_streamed_response_wrapper(
            named_schedules.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            named_schedules.update,
        )


class AsyncNamedSchedulesResourceWithStreamingResponse:
    def __init__(self, named_schedules: AsyncNamedSchedulesResource) -> None:
        self._named_schedules = named_schedules

        self.retrieve = async_to_streamed_response_wrapper(
            named_schedules.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            named_schedules.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["CustomerPreviewEventsParams", "Event"]


class CustomerPreviewEventsParams(TypedDict, total=False):
    customer_id: Required[str]

    events: Required[Iterable[Event]]
    """Array of usage events to include in the preview calculation.

    Must contain at least one event in `merge` mode.
    """

    mode: Literal["replace", "merge"]
    """Controls how the provided events are combined with existing usage data.

    Use `replace` to calculate the preview as if these are the only events for the
    customer, ignoring all historical usage. Use `merge` to combine these events
    with the customer's existing usage. Defaults to `replace`.
    """

    skip_zero_qty_line_items: bool
    """When `true`, line items with zero quantity are excluded from the response."""


class Event(TypedDict, total=False):
    event_type: Required[str]

    properties: Dict[str, object]

    timestamp: str
    """RFC 3339 formatted. If not provided, the current time will be used."""

    transaction_id: str
    """Optional unique identifier for event deduplication.

    When provided, preview events are automatically deduplicated against historical
    events from the past 34 days. Duplicate transaction IDs within the same request
    will return an error.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "Subscription",
    "BillingPeriods",
    "BillingPeriodsCurrent",
    "BillingPeriodsNext",
    "BillingPeriodsPrevious",
    "Proration",
    "QuantitySchedule",
    "SubscriptionRate",
    "SubscriptionRateProduct",
    "SeatConfig",
]


class BillingPeriodsCurrent(BaseModel):
    ending_before: datetime

    starting_at: datetime


class BillingPeriodsNext(BaseModel):
    ending_before: datetime

    starting_at: datetime


class BillingPeriodsPrevious(BaseModel):
    ending_before: datetime

    starting_at: datetime


class BillingPeriods(BaseModel):
    """Previous, current, and next billing periods for the subscription."""

    current: Optional[BillingPeriodsCurrent] = None

    next: Optional[BillingPeriodsNext] = None

    previous: Optional[BillingPeriodsPrevious] = None


class Proration(BaseModel):
    invoice_behavior: Literal["BILL_IMMEDIATELY", "BILL_ON_NEXT_COLLECTION_DATE"]

    is_prorated: bool


class QuantitySchedule(BaseModel):
    quantity: float

    starting_at: datetime

    ending_before: Optional[datetime] = None


class SubscriptionRateProduct(BaseModel):
    id: str

    name: str


class SubscriptionRate(BaseModel):
    billing_frequency: Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]

    product: SubscriptionRateProduct


class SeatConfig(BaseModel):
    seat_group_key: str
    """
    The property name, sent on usage events, that identifies the seat ID associated
    with the usage event. For example, the property name might be seat_id or
    user_id. The property must be set as a group key on billable metrics and a
    presentation/pricing group key on contract products. This allows linked
    recurring credits with an allocation per seat to be consumed by only one seat's
    usage.
    """


class Subscription(BaseModel):
    billing_periods: BillingPeriods
    """Previous, current, and next billing periods for the subscription."""

    collection_schedule: Literal["ADVANCE", "ARREARS"]

    proration: Proration

    quantity_management_mode: Literal["SEAT_BASED", "QUANTITY_ONLY"]
    """Determines how the subscription's quantity is controlled.

    Defaults to QUANTITY_ONLY. **QUANTITY_ONLY**: The subscription quantity is
    specified directly on the subscription. `initial_quantity` must be provided with
    this option. Compatible with recurring commits/credits that use POOLED
    allocation. **SEAT_BASED**: Use when you want to pass specific seat identifiers
    (e.g. add user_123) to increment and decrement a subscription quantity, rather
    than directly providing the quantity. You must use a **SEAT_BASED** subscription
    to use a linked recurring credit with an allocation per seat. `seat_config` must
    be provided with this option.
    """

    quantity_schedule: List[QuantitySchedule]
    """List of quantity schedule items for the subscription.

    Only includes the current quantity and future quantity changes.
    """

    starting_at: datetime

    subscription_rate: SubscriptionRate

    id: Optional[str] = None

    custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    description: Optional[str] = None

    ending_before: Optional[datetime] = None

    fiat_credit_type_id: Optional[str] = None

    name: Optional[str] = None

    seat_config: Optional[SeatConfig] = None
